<?php

/**
 * @file
 * Administrative settings callbacks for the Search Lucene Content module.
 */

/**
 * Administrative settings.
 *
 * @return
 *   An array of form elements.
 */
function luceneapi_node_admin_settings() {
  $form = array();

  // adds facet settings to admin form
  luceneapi_common_settings('luceneapi_node', $form);

  // name of the search tab
  $form['search']['luceneapi_node_search_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Search name'),
    '#default_value' => luceneapi_node_variable_get('search_name'),
    '#maxlength' => 64,
    '#description' => t(
      'Return value of <code>hook_search(&#039;name&#039;)</code>'
    ),
  );

  // strip core search from view
  $form['search']['luceneapi_node_hide_core_search'] = array(
    '#type' => 'radios',
    '#title' => t('Hide core search'),
    '#default_value' => luceneapi_node_variable_get('hide_core_search'),
    '#options' => array(
      1 => t('Yes'),
      0 => t('No'),
    ),
    '#description' => t(
      'Hides the core search, redirects core searches to luceneapi_node.'
    ),
  );

  // redirection
  $form['search']['luceneapi_node_redirect_code'] = array(
    '#type' => 'select',
    '#title' => t('Redirection status code'),
    '#default_value' => luceneapi_node_variable_get('redirect_code'),
    '#options' => array(
      301 => '301 - '. t('Moved permanently'),
      302 => '302 - '. t('Found'),
      307 => '307 - '. t('Temporary Redirect'),
    ),
    '#description' => t(
      'Status code used to redirect core search queries to luceneapi_node if core searches are hidden.'
    ),
  );

  // content type settings
  $form['type'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content types'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // description content type checkboxes
  $type_descrip = 'Adding content types to the exclude list will <b>delete'
                .' nodes of the selected content types from the index</b>.<br/>'
                .' Removing content types from the exclude list will <b>force'
                .' an index rebuild</b>.';

  // content types excluded from the index
  $form['type']['luceneapi_node_excluded_types'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Types excluded from index'),
    '#options' => array_map('check_plain', node_get_types('names')),
    '#default_value' => luceneapi_node_variable_get('excluded_types'),
    '#description' => t($type_descrip),
  );

  // adds submit handler to test if menu or index needs rebuilding
  $form['#submit'][] = 'luceneapi_node_admin_form_submit';

  return system_settings_form($form);
}

/**
 * If we choose to change the state of the hide core search, we notify Drupal
 * that the menu should be rebuilt on the next page load.
 *
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @param &$form_state
 *   An array containing the current state of the form.
 * @return string
 *   NULL
 */
function luceneapi_node_admin_form_submit($form, &$form_state) {
  $hide_core_search = $form_state['values']['luceneapi_node_hide_core_search'];
  if ($hide_core_search != luceneapi_node_variable_get('hide_core_search')) {
    variable_set('menu_rebuild_needed', TRUE);
  }

  // initializes flag for rebuild, array of content types to delete from index
  $rebuild_index = FALSE;
  $delete_types = array();

  // checks for changes in settings, gets types to be deleted, whether to rebuild index
  $cur_excluded = (array)luceneapi_node_variable_get('excluded_types');
  foreach ((array)$form_state['values']['luceneapi_node_excluded_types'] as $type => $selected) {
    if ($type) {
      if ($selected && !$cur_excluded[$type]) {
        $delete_types[] = $type;
      }
      elseif (!$rebuild_index && !$selected && $cur_excluded[$type]) {
        $rebuild_index = TRUE;
      }
    }
  }

  // deletes selected content types from the index
  if (!empty($delete_types) && $index = _luceneapi_node_open_index()) {
    if ($query = luceneapi_query_get('multiterm')) {
      foreach ($delete_types as $type) {
        if ($term = luceneapi_term_get($type, 'type')) {
          luceneapi_add_term($query, $term);
        }
      }
      $hits = $index->find($query);
      $clear_cache = (bool)count($hits);
      foreach ($hits as $hit) {
        $index->delete($hit->id);
      }
      if ($clear_cache) {
        try {
          $index->commit();
        }
        catch (Zend_Search_Exception $e) {
          luceneapi_throw_error($e);
        }
        _luceneapi_node_cache_clear();
      }
    }
  }

  // invokes hook_search('reset'), index will be rebuilt
  if ($rebuild_index) {
    luceneapi_node_search('reset');
    drupal_set_message(t('The Search Lucene Content index will be rebuilt.'));
  }
}
